@@ -3,3 +3,9 @@
 # The Wallet object is instantiated by the GUI
 
 # Notifications about network events are sent to the GUI by using network.register_callback()
+
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from . import qt
+    from . import kivy
