@@ -188,11 +188,12 @@ class ElectrumWindow(App, Logger):
     use_gossip = BooleanProperty(False)
     def on_use_gossip(self, instance, x):
         self.electrum_config.set_key('use_gossip', self.use_gossip, True)
-        if self.use_gossip:
-            self.network.start_gossip()
-        else:
-            self.network.run_from_another_thread(
-                self.network.stop_gossip())
+        if self.network:
+            if self.use_gossip:
+                self.network.start_gossip()
+            else:
+                self.network.run_from_another_thread(
+                    self.network.stop_gossip())
 
     use_change = BooleanProperty(False)
     def on_use_change(self, instance, x):
@@ -722,7 +723,8 @@ class ElectrumWindow(App, Logger):
         from .uix.dialogs.settings import SettingsDialog
         if self._settings_dialog is None:
             self._settings_dialog = SettingsDialog(self)
-        self._settings_dialog.update()
+        else:
+            self._settings_dialog.update()
         self._settings_dialog.open()
 
     def lightning_open_channel_dialog(self):
@@ -1027,7 +1029,7 @@ class ElectrumWindow(App, Logger):
         self.qr_dialog(label.name, label.data, show_text_with_qr)
 
     def show_error(self, error, width='200dp', pos=None, arrow_pos=None,
-                   exit=False, icon=f'atlas://{KIVY_GUI_PATH}/theming/light/error', duration=0,
+                   exit=False, icon=f'atlas://{KIVY_GUI_PATH}/theming/atlas/light/error', duration=0,
                    modal=False):
         ''' Show an error Message Bubble.
         '''
@@ -1039,7 +1041,7 @@ class ElectrumWindow(App, Logger):
                   exit=False, duration=0, modal=False):
         ''' Show an Info Message Bubble.
         '''
-        self.show_error(error, icon=f'atlas://{KIVY_GUI_PATH}/theming/light/important',
+        self.show_error(error, icon=f'atlas://{KIVY_GUI_PATH}/theming/atlas/light/important',
             duration=duration, modal=modal, exit=exit, pos=pos,
             arrow_pos=arrow_pos)
 
@@ -1080,7 +1082,7 @@ class ElectrumWindow(App, Logger):
             info_bubble.show_arrow = False
             img.allow_stretch = True
             info_bubble.dim_background = True
-            info_bubble.background_image = f'atlas://{KIVY_GUI_PATH}/theming/light/card'
+            info_bubble.background_image = f'atlas://{KIVY_GUI_PATH}/theming/atlas/light/card'
         else:
             info_bubble.fs = False
             info_bubble.icon = icon
@@ -1182,7 +1184,8 @@ class ElectrumWindow(App, Logger):
         from .uix.dialogs.addresses import AddressesDialog
         if self._addresses_dialog is None:
             self._addresses_dialog = AddressesDialog(self)
-        self._addresses_dialog.update()
+        else:
+            self._addresses_dialog.update()
         self._addresses_dialog.open()
 
     def fee_dialog(self):
@@ -1334,7 +1337,6 @@ class ElectrumWindow(App, Logger):
                 self.show_error(_("Backup NOT saved. Backup directory not configured."))
             return
 
-        backup_dir = util.android_backup_dir()
         from android.permissions import request_permissions, Permission
         def cb(permissions, grant_results: Sequence[bool]):
             if not grant_results or not grant_results[0]:
@@ -1342,6 +1344,7 @@ class ElectrumWindow(App, Logger):
                 return
             # note: Clock.schedule_once is a hack so that we get called on a non-daemon thread
             #       (needed for WalletDB.write)
+            backup_dir = util.android_backup_dir()
             Clock.schedule_once(lambda dt: self._save_backup(backup_dir))
         request_permissions([Permission.WRITE_EXTERNAL_STORAGE], cb)
 
