@@ -4,19 +4,20 @@ from typing import TYPE_CHECKING, Optional, Union
 from kivy.lang import Builder
 from kivy.factory import Factory
 from kivy.uix.popup import Popup
-from .fee_dialog import FeeDialog
 
 from electrum.util import bh2u
 from electrum.logging import Logger
 from electrum.lnutil import LOCAL, REMOTE, format_short_channel_id
 from electrum.lnchannel import AbstractChannel, Channel, ChannelState
 from electrum.gui.kivy.i18n import _
-from .question import Question
 from electrum.transaction import PartialTxOutput, Transaction
 from electrum.util import NotEnoughFunds, NoDynamicFeeEstimates, format_fee_satoshis, quantize_feerate
 from electrum.lnutil import ln_dummy_address
 from electrum.gui import messages
 
+from ..actiondropdown import ActionButtonOption, ActionDropdown
+from .fee_dialog import FeeDialog
+from .question import Question
 from .qr_dialog import QRDialog
 from .choice_dialog import ChoiceDialog
 
@@ -172,6 +173,7 @@ Builder.load_string(r'''
 <LightningChannelsDialog@Popup>:
     name: 'lightning_channels'
     title: _('Lightning Network')
+    has_network: False
     has_lightning: False
     has_gossip: False
     can_send: ''
@@ -205,13 +207,13 @@ Builder.load_string(r'''
                 size_hint: 0.3, None
                 height: '48dp'
                 text: _('Open Channel')
-                disabled: not root.has_lightning
+                disabled: not (root.has_network and root.has_lightning)
                 on_release: popup.app.popup_dialog('lightning_open_channel_dialog')
             Button:
                 size_hint: 0.3, None
                 height: '48dp'
                 text: _('Swap')
-                disabled: not root.has_lightning
+                disabled: not (root.has_network and root.has_lightning)
                 on_release: popup.app.popup_dialog('swap_dialog')
             Button:
                 size_hint: 0.3, None
@@ -242,6 +244,8 @@ Builder.load_string(r'''
     can_receive:''
     is_open:False
     warning: ''
+    is_frozen_for_sending: False
+    is_frozen_for_receiving: False
     BoxLayout:
         padding: '12dp', '12dp', '12dp', '12dp'
         spacing: '12dp'
@@ -284,6 +288,12 @@ Builder.load_string(r'''
                 BoxLabel:
                     text: _('Fee rate')
                     value: '{} sat/byte'.format(root.feerate)
+                BoxLabel:
+                    text: _('Frozen (for sending)')
+                    value: str(root.is_frozen_for_sending)
+                BoxLabel:
+                    text: _('Frozen (for receiving)')
+                    value: str(root.is_frozen_for_receiving)
                 Widget:
                     size_hint: 1, 0.1
                 TopLabel:
@@ -312,29 +322,12 @@ Builder.load_string(r'''
         BoxLayout:
             size_hint: 1, None
             height: '48dp'
-            Button:
+            ActionDropdown:
+                id: action_dropdown
                 size_hint: 0.5, None
                 height: '48dp'
-                text: _('Backup')
-                on_release: root.export_backup()
-            Button:
+            Widget:
                 size_hint: 0.5, None
-                height: '48dp'
-                text: _('Close')
-                on_release: root.close()
-                disabled: root.is_closed
-            Button:
-                size_hint: 0.5, None
-                height: '48dp'
-                text: _('Force-close')
-                on_release: root.force_close()
-                disabled: root.is_closed
-            Button:
-                size_hint: 0.5, None
-                height: '48dp'
-                text: _('Delete')
-                on_release: root.remove_channel()
-                disabled: not root.can_be_deleted
 
 <ChannelBackupPopup@Popup>:
     id: popuproot
@@ -487,12 +480,30 @@ class ChannelDetailsPopup(Popup, Logger):
         closed = chan.get_closing_height()
         if closed:
             self.closing_txid, closing_height, closing_timestamp = closed
-        msg = ' '.join([
-            _("Trampoline routing is enabled, but this channel is with a non-trampoline node."),
-            _("This channel may still be used for receiving, but it is frozen for sending."),
-            _("If you want to keep using this channel, you need to disable trampoline routing in your preferences."),
-        ])
+        msg = messages.MSG_NON_TRAMPOLINE_CHANNEL_FROZEN_WITHOUT_GOSSIP
         self.warning = '' if self.app.wallet.lnworker.channel_db or self.app.wallet.lnworker.is_trampoline_peer(chan.node_id) else _('Warning') + ': ' + msg
+        self.is_frozen_for_sending = chan.is_frozen_for_sending()
+        self.is_frozen_for_receiving = chan.is_frozen_for_receiving()
+        self.update_action_dropdown()
+
+    def update_action_dropdown(self):
+        action_dropdown = self.ids.action_dropdown  # type: ActionDropdown
+        options = [
+            ActionButtonOption(text=_('Backup'), func=lambda btn: self.export_backup()),
+            ActionButtonOption(text=_('Close channel'), func=lambda btn: self.close(), enabled=not self.is_closed),
+            ActionButtonOption(text=_('Force-close'), func=lambda btn: self.force_close(), enabled=not self.is_closed),
+            ActionButtonOption(text=_('Delete'), func=lambda btn: self.remove_channel(), enabled=self.can_be_deleted),
+        ]
+        if not self.chan.is_closed():
+            if not self.chan.is_frozen_for_sending():
+                options.append(ActionButtonOption(text=_("Freeze") + "\n(for sending)", func=lambda btn: self.freeze_for_sending()))
+            else:
+                options.append(ActionButtonOption(text=_("Unfreeze") + "\n(for sending)", func=lambda btn: self.freeze_for_sending()))
+            if not self.chan.is_frozen_for_receiving():
+                options.append(ActionButtonOption(text=_("Freeze") + "\n(for receiving)", func=lambda btn: self.freeze_for_receiving()))
+            else:
+                options.append(ActionButtonOption(text=_("Unfreeze") + "\n(for receiving)", func=lambda btn: self.freeze_for_receiving()))
+        action_dropdown.update(options=options)
 
     def close(self):
         dialog = ChoiceDialog(
@@ -582,6 +593,20 @@ class ChannelDetailsPopup(Popup, Logger):
             self.logger.exception("Could not force close channel")
             self.app.show_info(_('Could not force close channel: ') + repr(e)) # repr because str(Exception()) == ''
 
+    def freeze_for_sending(self):
+        lnworker = self.chan.lnworker
+        if lnworker.channel_db or lnworker.is_trampoline_peer(self.chan.node_id):
+            self.is_frozen_for_sending = not self.is_frozen_for_sending
+            self.chan.set_frozen_for_sending(self.is_frozen_for_sending)
+            self.update_action_dropdown()
+        else:
+            self.app.show_info(messages.MSG_NON_TRAMPOLINE_CHANNEL_FROZEN_WITHOUT_GOSSIP)
+
+    def freeze_for_receiving(self):
+        self.is_frozen_for_receiving = not self.is_frozen_for_receiving
+        self.chan.set_frozen_for_receiving(self.is_frozen_for_receiving)
+        self.update_action_dropdown()
+
 
 class LightningChannelsDialog(Factory.Popup):
 
@@ -589,8 +614,9 @@ class LightningChannelsDialog(Factory.Popup):
         super(LightningChannelsDialog, self).__init__()
         self.clocks = []
         self.app = app
+        self.has_network = bool(self.app.network)
         self.has_lightning = app.wallet.has_lightning()
-        self.has_gossip = self.app.network.channel_db is not None
+        self.has_gossip = self.has_network and self.app.network.channel_db is not None
         self.update()
 
     def show_item(self, obj):
@@ -678,12 +704,12 @@ class SwapDialog(Factory.Popup):
 
     def update_fee_text(self):
         fee_per_kb = self.config.fee_per_kb()
+        fee_per_b = format_fee_satoshis(fee_per_kb / 1000) if fee_per_kb is not None else "unknown"
         # eta is -1 when block inclusion cannot be estimated for low fees
         eta = self.config.fee_to_eta(fee_per_kb)
 
-        fee_per_b = format_fee_satoshis(fee_per_kb / 1000)
         suggest_fee = self.config.eta_target_to_fee(RECOMMEND_BLOCKS_SWAP)
-        suggest_fee_per_b = format_fee_satoshis(suggest_fee / 1000)
+        suggest_fee_per_b = format_fee_satoshis(suggest_fee / 1000) if suggest_fee is not None else "unknown"
 
         s = 's' if eta > 1 else ''
         if eta > RECOMMEND_BLOCKS_SWAP or eta == -1:
