@@ -49,7 +49,9 @@ try:
             except:
                 return False
 
-except ImportError:
+except ImportError as e:
+    if not (isinstance(e, ModuleNotFoundError) and e.name == 'ckcc'):
+        _logger.exception('error importing coldcard plugin deps')
     requirements_ok = False
 
     COINKITE_VID = 0xd13e
@@ -95,7 +97,7 @@ class CKCCClient(HardwareClientBase):
         if expected_xpub is None:
             expected_xpub = self.dev.master_xpub
 
-        if ( (self._expected_device is not None) 
+        if ((self._expected_device is not None)
                 or (self.dev.master_fingerprint != expected_xfp)
                 or (self.dev.master_xpub != expected_xpub)):
             # probably indicating programing error, not hacking
@@ -142,19 +144,19 @@ class CKCCClient(HardwareClientBase):
         else:
             lab = 'Coldcard ' + xfp2str(self.dev.master_fingerprint)
 
-        # Hack zone: during initial setup I need the xfp and master xpub but 
-        # very few objects are passed between the various steps of base_wizard.
-        # Solution: return a string with some hidden metadata
-        # - see <https://stackoverflow.com/questions/7172772/abc-for-string>
-        # - needs to work w/ deepcopy
-        class LabelStr(str):
-            def __new__(cls, s, xfp=None, xpub=None):
-                self = super().__new__(cls, str(s))
-                self.xfp = getattr(s, 'xfp', xfp)
-                self.xpub = getattr(s, 'xpub', xpub)
-                return self
+        return lab
 
-        return LabelStr(lab, self.dev.master_fingerprint, self.dev.master_xpub)
+    def manipulate_keystore_dict_during_wizard_setup(self, d: dict):
+        master_xpub = self.dev.master_xpub
+        if master_xpub is not None:
+            try:
+                node = BIP32Node.from_xkey(master_xpub)
+            except InvalidMasterKeyVersionBytes:
+                raise UserFacingException(
+                    _('Invalid xpub magic. Make sure your {} device is set to the correct chain.').format(self.device) + ' ' +
+                    _('You might have to unplug and plug it in again.')
+                ) from None
+            d['ckcc_xpub'] = master_xpub
 
     @runs_in_hwd_thread
     def has_usable_connection_with_device(self):
@@ -262,8 +264,7 @@ class Coldcard_KeyStore(Hardware_KeyStore):
         # we need to know at least the fingerprint of the master xpub to verify against MiTM
         # - device reports these value during encryption setup process
         # - full xpub value now optional
-        lab = d['label']
-        self.ckcc_xpub = getattr(lab, 'xpub', None) or d.get('ckcc_xpub', None)
+        self.ckcc_xpub = d.get('ckcc_xpub', None)
 
     def dump(self):
         # our additions to the stored data about keystore -- only during creation?
@@ -568,31 +569,24 @@ class ColdcardPlugin(HW_PluginBase):
         print('# Exported from Electrum', file=fp)
         print(f'Name: {name:.20s}', file=fp)
         print(f'Policy: {wallet.m} of {wallet.n}', file=fp)
-        print(f'Format: {wallet.txin_type.upper()}' , file=fp)
+        print(f'Format: {wallet.txin_type.upper()}', file=fp)
 
         xpubs = []
-        derivs = set()
         for xpub, ks in zip(wallet.get_master_public_keys(), wallet.get_keystores()):  # type: str, KeyStoreWithMPK
             fp_bytes, der_full = ks.get_fp_and_derivation_to_be_used_in_partial_tx(der_suffix=[], only_der_suffix=False)
             fp_hex = fp_bytes.hex().upper()
             der_prefix_str = bip32.convert_bip32_intpath_to_strpath(der_full)
-            xpubs.append( (fp_hex, xpub, der_prefix_str) )
-            derivs.add(der_prefix_str)
+            xpubs.append((fp_hex, xpub, der_prefix_str))
 
-        # Derivation doesn't matter too much to the Coldcard, since it
-        # uses key path data from PSBT or USB request as needed. However,
-        # if there is a clear value, provide it.
-        if len(derivs) == 1:
-            print("Derivation: " + derivs.pop(), file=fp)
+        # Before v3.2.1 derivation didn't matter too much to the Coldcard, since it
+        # could use key path data from PSBT or USB request as needed. However,
+        # derivation data is now required.
 
         print('', file=fp)
 
         assert len(xpubs) == wallet.n
         for xfp, xpub, der_prefix in xpubs:
-            if derivs:
-                # show as a comment if unclear
-                print(f'# derivation: {der_prefix}', file=fp)
-
+            print(f'Derivation: {der_prefix}', file=fp)
             print(f'{xfp}: {xpub}\n', file=fp)
 
     def show_address(self, wallet, address, keystore: 'Coldcard_KeyStore' = None):
