@@ -7,7 +7,7 @@ from typing import Optional, Tuple
 from electrum import ecc
 from electrum import bip32
 from electrum.crypto import hash_160
-from electrum.bitcoin import int_to_hex, var_int, is_segwit_script_type
+from electrum.bitcoin import int_to_hex, var_int, is_segwit_script_type, is_b58_address
 from electrum.bip32 import BIP32Node, convert_bip32_intpath_to_strpath
 from electrum.i18n import _
 from electrum.keystore import Hardware_KeyStore
@@ -16,7 +16,7 @@ from electrum.wallet import Standard_Wallet
 from electrum.util import bfh, bh2u, versiontuple, UserFacingException
 from electrum.base_wizard import ScriptTypeNotSupported
 from electrum.logging import get_logger
-from electrum.plugin import runs_in_hwd_thread
+from electrum.plugin import runs_in_hwd_thread, Device
 
 from ..hw_wallet import HW_PluginBase, HardwareClientBase
 from ..hw_wallet.plugin import is_any_tx_output_on_change_branch, validate_op_return_output, LibraryFoundButUnusable
@@ -35,7 +35,9 @@ try:
     from btchip.btchipException import BTChipException
     BTCHIP = True
     BTCHIP_DEBUG = False
-except ImportError:
+except ImportError as e:
+    if not (isinstance(e, ModuleNotFoundError) and e.name == 'btchip'):
+        _logger.exception('error importing ledger plugin deps')
     BTCHIP = False
 
 MSG_NEEDS_FW_UPDATE_GENERIC = _('Firmware version too old. Please update at') + \
@@ -95,15 +97,7 @@ class Ledger_Client(HardwareClientBase):
         return self._product_key[0] == 0x2581
 
     def device_model_name(self):
-        if self.is_hw1():
-            return "Ledger HW.1"
-        if self._product_key == (0x2c97, 0x0000):
-            return "Ledger Blue"
-        if self._product_key == (0x2c97, 0x0001):
-            return "Ledger Nano S"
-        if self._product_key == (0x2c97, 0x0004):
-            return "Ledger Nano X"
-        return None
+        return LedgerPlugin.device_name_from_product_key(self._product_key)
 
     @runs_in_hwd_thread
     def has_usable_connection_with_device(self):
@@ -199,7 +193,7 @@ class Ledger_Client(HardwareClientBase):
             except BTChipException as e:
                 if (e.sw == 0x6985):
                     self.close()
-                    self.handler.get_setup( )
+                    self.handler.get_setup()
                     # Acquire the new client on the next run
                 else:
                     raise e
@@ -300,18 +294,20 @@ class Ledger_KeyStore(Hardware_KeyStore):
         message = message.encode('utf8')
         message_hash = hashlib.sha256(message).hexdigest().upper()
         # prompt for the PIN before displaying the dialog if necessary
-        client = self.get_client()
+        client_ledger = self.get_client()
+        client_electrum = self.get_client_electrum()
         address_path = self.get_derivation_prefix()[2:] + "/%d/%d"%sequence
         self.handler.show_message("Signing message ...\r\nMessage hash: "+message_hash)
         try:
-            info = self.get_client().signMessagePrepare(address_path, message)
+            info = client_ledger.signMessagePrepare(address_path, message)
             pin = ""
             if info['confirmationNeeded']:
-                pin = self.handler.get_auth( info ) # does the authenticate dialog and returns pin
+                # do the authenticate dialog and get pin:
+                pin = self.handler.get_auth(info, client=client_electrum)
                 if not pin:
                     raise UserWarning(_('Cancelled by user'))
                 pin = str(pin).encode()
-            signature = self.get_client().signMessageSign(pin)
+            signature = client_ledger.signMessageSign(pin)
         except BTChipException as e:
             if e.sw == 0x6a80:
                 self.give_error("Unfortunately, this message cannot be signed by the Ledger wallet. Only alphanumerical messages shorter than 140 characters are supported. Please remove any extra characters (tab, carriage return) and retry.")
@@ -338,7 +334,12 @@ class Ledger_KeyStore(Hardware_KeyStore):
         if sLength == 33:
             s = s[1:]
         # And convert it
-        return bytes([27 + 4 + (signature[0] & 0x01)]) + r + s
+
+        # Pad r and s points with 0x00 bytes when the point is small to get valid signature.
+        r_padded = bytes([0x00]) * (32 - len(r)) + r
+        s_padded = bytes([0x00]) * (32 - len(s)) + s
+        
+        return bytes([27 + 4 + (signature[0] & 0x01)]) + r_padded + s_padded
 
     @runs_in_hwd_thread
     @test_pin_unlocked
@@ -384,7 +385,7 @@ class Ledger_KeyStore(Hardware_KeyStore):
 
             redeemScript = Transaction.get_preimage_script(txin)
             txin_prev_tx = txin.utxo
-            if txin_prev_tx is None and not Transaction.is_segwit_input(txin):
+            if txin_prev_tx is None and not txin.is_segwit():
                 raise UserFacingException(_('Missing previous tx for legacy input.'))
             txin_prev_tx_raw = txin_prev_tx.serialize() if txin_prev_tx else None
             inputs.append([txin_prev_tx_raw,
@@ -414,6 +415,8 @@ class Ledger_KeyStore(Hardware_KeyStore):
             if len(tx.outputs()) > 2:
                 self.give_error("Transaction with more than 2 outputs not supported")
         for txout in tx.outputs():
+            if client_electrum.is_hw1() and txout.address and not is_b58_address(txout.address):
+                self.give_error(_("This {} device can only send to base58 addresses.").format(self.device))
             if not txout.address:
                 if client_electrum.is_hw1():
                     self.give_error(_("Only address outputs are supported by {}").format(self.device))
@@ -484,12 +487,13 @@ class Ledger_KeyStore(Hardware_KeyStore):
                 if outputData['confirmationNeeded']:
                     outputData['address'] = output
                     self.handler.finished()
-                    pin = self.handler.get_auth( outputData ) # does the authenticate dialog and returns pin
+                    # do the authenticate dialog and get pin:
+                    pin = self.handler.get_auth(outputData, client=client_electrum)
                     if not pin:
                         raise UserWarning()
                     self.handler.show_message(_("Confirmed. Signing Transaction..."))
                 while inputIndex < len(inputs):
-                    singleInput = [ chipInputs[inputIndex] ]
+                    singleInput = [chipInputs[inputIndex]]
                     client_ledger.startUntrustedTransaction(False, 0,
                                                             singleInput, redeemScripts[inputIndex], version=tx.version)
                     inputSignature = client_ledger.untrustedHashSign(inputsPaths[inputIndex], pin, lockTime=tx.locktime)
@@ -510,7 +514,8 @@ class Ledger_KeyStore(Hardware_KeyStore):
                     if outputData['confirmationNeeded']:
                         outputData['address'] = output
                         self.handler.finished()
-                        pin = self.handler.get_auth( outputData ) # does the authenticate dialog and returns pin
+                        # do the authenticate dialog and get pin:
+                        pin = self.handler.get_auth(outputData, client=client_electrum)
                         if not pin:
                             raise UserWarning()
                         self.handler.show_message(_("Confirmed. Signing Transaction..."))
@@ -573,7 +578,7 @@ class Ledger_KeyStore(Hardware_KeyStore):
 
 class LedgerPlugin(HW_PluginBase):
     keystore_class = Ledger_KeyStore
-    minimum_library = (0, 1, 30)
+    minimum_library = (0, 1, 32)
     client = None
     DEVICE_IDS = [
                    (0x2581, 0x1807), # HW.1 legacy btchip
@@ -590,6 +595,11 @@ class LedgerPlugin(HW_PluginBase):
                    (0x2c97, 0x0009), # RFU
                    (0x2c97, 0x000a)  # RFU
                  ]
+    VENDOR_IDS = (0x2c97,)
+    LEDGER_MODEL_IDS = {
+        0x10: "Ledger Nano S",
+        0x40: "Ledger Nano X",
+    }
     SUPPORTED_XTYPES = ('standard', 'p2wpkh-p2sh', 'p2wpkh', 'p2wsh-p2sh', 'p2wsh')
 
     def __init__(self, parent, config, name):
@@ -598,7 +608,10 @@ class LedgerPlugin(HW_PluginBase):
         self.libraries_available = self.check_libraries_available()
         if not self.libraries_available:
             return
+        # to support legacy devices and legacy firmwares
         self.device_manager().register_devices(self.DEVICE_IDS, plugin=self)
+        # to support modern firmware
+        self.device_manager().register_vendor_ids(self.VENDOR_IDS, plugin=self)
 
     def get_library_version(self):
         try:
@@ -613,6 +626,43 @@ class LedgerPlugin(HW_PluginBase):
         else:
             raise LibraryFoundButUnusable(library_version=version)
 
+    @classmethod
+    def _recognize_device(cls, product_key) -> Tuple[bool, Optional[str]]:
+        """Returns (can_recognize, model_name) tuple."""
+        # legacy product_keys
+        if product_key in cls.DEVICE_IDS:
+            if product_key[0] == 0x2581:
+                return True, "Ledger HW.1"
+            if product_key == (0x2c97, 0x0000):
+                return True, "Ledger Blue"
+            if product_key == (0x2c97, 0x0001):
+                return True, "Ledger Nano S"
+            if product_key == (0x2c97, 0x0004):
+                return True, "Ledger Nano X"
+            return True, None
+        # modern product_keys
+        if product_key[0] == 0x2c97:
+            product_id = product_key[1]
+            model_id = product_id >> 8
+            if model_id in cls.LEDGER_MODEL_IDS:
+                model_name = cls.LEDGER_MODEL_IDS[model_id]
+                return True, model_name
+        # give up
+        return False, None
+
+    def can_recognize_device(self, device: Device) -> bool:
+        return self._recognize_device(device.product_key)[0]
+
+    @classmethod
+    def device_name_from_product_key(cls, product_key) -> Optional[str]:
+        return cls._recognize_device(product_key)[1]
+
+    def create_device_from_hid_enumeration(self, d, *, product_key):
+        device = super().create_device_from_hid_enumeration(d, product_key=product_key)
+        if not self.can_recognize_device(device):
+            return None
+        return device
+
     @runs_in_hwd_thread
     def get_btchip_device(self, device):
         ledger = False
