@@ -116,7 +116,7 @@ class Plugin(RevealerPlugin):
         bcreate.setMaximumWidth(181)
         bcreate.setDefault(True)
         vbox.addWidget(bcreate, Qt.AlignCenter)
-        self.load_noise = ScanQRTextEdit()
+        self.load_noise = ScanQRTextEdit(config=self.config)
         self.load_noise.setTabChangesFocus(True)
         self.load_noise.textChanged.connect(self.on_edit)
         self.load_noise.setMaximumHeight(33)
@@ -137,7 +137,7 @@ class Plugin(RevealerPlugin):
             try:
                 self.make_digital(self.d)
             except Exception:
-                traceback.print_exc(file=sys.stdout)
+                self.logger.exception('')
             else:
                 self.cypherseed_dialog(window)
 
@@ -253,7 +253,7 @@ class Plugin(RevealerPlugin):
         self.vbox.addWidget(cprint)
         self.vbox.addSpacing(1)
         self.vbox.addWidget(WWLabel("<b>"+_("OR")+"</b> "+_("type a custom alphanumerical secret below:")))
-        self.text = ScanQRTextEdit()
+        self.text = ScanQRTextEdit(config=self.config)
         self.text.setTabChangesFocus(True)
         self.text.setMaximumHeight(70)
         self.text.textChanged.connect(self.customtxt_limits)
@@ -299,7 +299,7 @@ class Plugin(RevealerPlugin):
         bitmap.fill(Qt.white)
         painter = QPainter()
         painter.begin(bitmap)
-        QFontDatabase.addApplicationFont(os.path.join(os.path.dirname(__file__), 'SourceSansPro-Bold.otf') )
+        QFontDatabase.addApplicationFont(os.path.join(os.path.dirname(__file__), 'SourceSansPro-Bold.otf'))
         if len(txt) < 102 :
             fontsize = 15
             linespace = 15
@@ -325,7 +325,7 @@ class Plugin(RevealerPlugin):
             while len(' '.join(map(str, temp_seed))) > max_letters:
                nwords = nwords - 1
                temp_seed = seed_array[:nwords]
-            painter.drawText(QRect(0, linespace*n , self.SIZE[0], self.SIZE[1]), Qt.AlignHCenter, ' '.join(map(str, temp_seed)))
+            painter.drawText(QRect(0, linespace*n, self.SIZE[0], self.SIZE[1]), Qt.AlignHCenter, ' '.join(map(str, temp_seed)))
             del seed_array[:nwords]
 
         painter.end()
@@ -463,7 +463,7 @@ class Plugin(RevealerPlugin):
         painter.end()
 
     def pixelcode_2x2(self, img):
-        result = QImage(img.width()*2, img.height()*2, QImage.Format_ARGB32 )
+        result = QImage(img.width()*2, img.height()*2, QImage.Format_ARGB32)
         white = qRgba(255,255,255,0)
         black = qRgba(0,0,0,255)
 
@@ -625,7 +625,7 @@ class Plugin(RevealerPlugin):
                                      + self.versioned_seed.checksum)
                 target = QRectF(base_img.width()-65-qr_size,
                                 base_img.height()-65-qr_size,
-                                qr_size, qr_size )
+                                qr_size, qr_size)
                 painter.drawImage(target, qr_qt)
                 painter.setPen(QPen(Qt.black, 4))
                 painter.drawLine(base_img.width()-65-qr_size,
